# Generated by ariadne-codegen
# Source: openhexa/graphql/queries.graphql

from typing import List

from pydantic import Field

from .base_model import BaseModel
from .enums import RemoveFromFavoritesError


class RemoveWebappFromFavorites(BaseModel):
    remove_from_favorites: "RemoveWebappFromFavoritesRemoveFromFavorites" = Field(
        alias="removeFromFavorites"
    )


class RemoveWebappFromFavoritesRemoveFromFavorites(BaseModel):
    success: bool
    errors: List[RemoveFromFavoritesError]


RemoveWebappFromFavorites.model_rebuild()
