# Generated by ariadne-codegen
# Source: openhexa/graphql/queries.graphql

from typing import Any, List, Optional

from pydantic import Field

from .base_model import BaseModel
from .enums import InviteWorkspaceMembershipError


class InviteWorkspaceMember(BaseModel):
    invite_workspace_member: "InviteWorkspaceMemberInviteWorkspaceMember" = Field(
        alias="inviteWorkspaceMember"
    )


class InviteWorkspaceMemberInviteWorkspaceMember(BaseModel):
    success: bool
    errors: List[InviteWorkspaceMembershipError]
    workspace_membership: Optional[
        "InviteWorkspaceMemberInviteWorkspaceMemberWorkspaceMembership"
    ] = Field(alias="workspaceMembership")


class InviteWorkspaceMemberInviteWorkspaceMemberWorkspaceMembership(BaseModel):
    id: Any


InviteWorkspaceMember.model_rebuild()
InviteWorkspaceMemberInviteWorkspaceMember.model_rebuild()
