# Generated by ariadne-codegen
# Source: openhexa/graphql/queries.graphql

from typing import Any, List, Optional

from pydantic import Field

from .base_model import BaseModel
from .enums import CreateWebappError


class CreateWebapp(BaseModel):
    create_webapp: "CreateWebappCreateWebapp" = Field(alias="createWebapp")


class CreateWebappCreateWebapp(BaseModel):
    success: bool
    errors: List[CreateWebappError]
    webapp: Optional["CreateWebappCreateWebappWebapp"]


class CreateWebappCreateWebappWebapp(BaseModel):
    id: Any


CreateWebapp.model_rebuild()
CreateWebappCreateWebapp.model_rebuild()
