# Generated by ariadne-codegen
# Source: openhexa/graphql/queries.graphql

from typing import List

from pydantic import Field

from .base_model import BaseModel
from .enums import PipelineError


class UploadPipeline(BaseModel):
    upload_pipeline: "UploadPipelineUploadPipeline" = Field(alias="uploadPipeline")


class UploadPipelineUploadPipeline(BaseModel):
    success: bool
    errors: List[PipelineError]


UploadPipeline.model_rebuild()
