# Generated by ariadne-codegen
# Source: openhexa/graphql/queries.graphql

from typing import List

from pydantic import Field

from .base_model import BaseModel
from .enums import ArchiveWorkspaceError


class ArchiveWorkspace(BaseModel):
    archive_workspace: "ArchiveWorkspaceArchiveWorkspace" = Field(
        alias="archiveWorkspace"
    )


class ArchiveWorkspaceArchiveWorkspace(BaseModel):
    success: bool
    errors: List[ArchiveWorkspaceError]


ArchiveWorkspace.model_rebuild()
