# Generated by ariadne-codegen
# Source: openhexa/graphql/queries.graphql

from typing import List, Optional

from pydantic import Field

from .base_model import BaseModel


class Workspaces(BaseModel):
    workspaces: "WorkspacesWorkspaces"


class WorkspacesWorkspaces(BaseModel):
    total_pages: int = Field(alias="totalPages")
    items: List["WorkspacesWorkspacesItems"]


class WorkspacesWorkspacesItems(BaseModel):
    slug: str
    name: str
    description: Optional[str]
    countries: List["WorkspacesWorkspacesItemsCountries"]


class WorkspacesWorkspacesItemsCountries(BaseModel):
    code: str
    alpha_3: str = Field(alias="alpha3")
    name: str


Workspaces.model_rebuild()
WorkspacesWorkspaces.model_rebuild()
WorkspacesWorkspacesItems.model_rebuild()
