# Generated by ariadne-codegen
# Source: openhexa/graphql/queries.graphql

from typing import List

from pydantic import Field

from .base_model import BaseModel
from .enums import UpgradePipelineVersionFromTemplateError


class UpgradePipelineVersionFromTemplate(BaseModel):
    upgrade_pipeline_version_from_template: (
        "UpgradePipelineVersionFromTemplateUpgradePipelineVersionFromTemplate"
    ) = Field(alias="upgradePipelineVersionFromTemplate")


class UpgradePipelineVersionFromTemplateUpgradePipelineVersionFromTemplate(BaseModel):
    success: bool
    errors: List[UpgradePipelineVersionFromTemplateError]


UpgradePipelineVersionFromTemplate.model_rebuild()
