# Generated by ariadne-codegen
# Source: openhexa/graphql/queries.graphql

from typing import List

from pydantic import Field

from .base_model import BaseModel
from .enums import DeleteDatasetError


class DeleteDataset(BaseModel):
    delete_dataset: "DeleteDatasetDeleteDataset" = Field(alias="deleteDataset")


class DeleteDatasetDeleteDataset(BaseModel):
    success: bool
    errors: List[DeleteDatasetError]


DeleteDataset.model_rebuild()
