# Generated by ariadne-codegen
# Source: openhexa/graphql/queries.graphql

from typing import Any, List, Optional

from pydantic import Field

from .base_model import BaseModel


class Datasets(BaseModel):
    datasets: "DatasetsDatasets"


class DatasetsDatasets(BaseModel):
    total_pages: int = Field(alias="totalPages")
    items: List["DatasetsDatasetsItems"]


class DatasetsDatasetsItems(BaseModel):
    id: str
    slug: str
    name: str
    description: Optional[str]
    created_at: Any = Field(alias="createdAt")
    updated_at: Any = Field(alias="updatedAt")
    created_by: Optional["DatasetsDatasetsItemsCreatedBy"] = Field(alias="createdBy")
    permissions: "DatasetsDatasetsItemsPermissions"


class DatasetsDatasetsItemsCreatedBy(BaseModel):
    id: Any
    display_name: str = Field(alias="displayName")


class DatasetsDatasetsItemsPermissions(BaseModel):
    update: bool
    delete: bool


Datasets.model_rebuild()
DatasetsDatasets.model_rebuild()
DatasetsDatasetsItems.model_rebuild()
