# Generated by ariadne-codegen
# Source: openhexa/graphql/queries.graphql

from typing import List

from pydantic import Field

from .base_model import BaseModel
from .enums import PipelineError


class StopPipeline(BaseModel):
    stop_pipeline: "StopPipelineStopPipeline" = Field(alias="stopPipeline")


class StopPipelineStopPipeline(BaseModel):
    success: bool
    errors: List[PipelineError]


StopPipeline.model_rebuild()
