# Generated by ariadne-codegen
# Source: openhexa/graphql/queries.graphql

from typing import Any, List, Optional

from pydantic import Field

from .base_model import BaseModel


class Workspace(BaseModel):
    workspace: Optional["WorkspaceWorkspace"]


class WorkspaceWorkspace(BaseModel):
    slug: str
    name: str
    description: Optional[str]
    docker_image: Optional[str] = Field(alias="dockerImage")
    configuration: Optional[Any]
    countries: List["WorkspaceWorkspaceCountries"]
    permissions: "WorkspaceWorkspacePermissions"


class WorkspaceWorkspaceCountries(BaseModel):
    code: str
    flag: str
    name: str
    alpha_3: str = Field(alias="alpha3")


class WorkspaceWorkspacePermissions(BaseModel):
    delete: bool
    update: bool
    manage_members: bool = Field(alias="manageMembers")


Workspace.model_rebuild()
WorkspaceWorkspace.model_rebuild()
