# Generated by ariadne-codegen
# Source: openhexa/graphql/queries.graphql

from typing import List, Optional

from pydantic import Field

from .base_model import BaseModel


class GetCountries(BaseModel):
    workspace: Optional["GetCountriesWorkspace"]


class GetCountriesWorkspace(BaseModel):
    countries: List["GetCountriesWorkspaceCountries"]


class GetCountriesWorkspaceCountries(BaseModel):
    code: str
    name: str
    flag: str
    alpha_3: str = Field(alias="alpha3")


GetCountries.model_rebuild()
GetCountriesWorkspace.model_rebuild()
