# Generated by ariadne-codegen
# Source: openhexa/graphql/queries.graphql

from typing import Any, List

from pydantic import Field

from .base_model import BaseModel


class GetUsers(BaseModel):
    users: List["GetUsersUsers"]


class GetUsersUsers(BaseModel):
    id: Any
    email: str
    display_name: str = Field(alias="displayName")
    avatar: "GetUsersUsersAvatar"


class GetUsersUsersAvatar(BaseModel):
    initials: str
    color: str


GetUsers.model_rebuild()
GetUsersUsers.model_rebuild()
