# Generated by ariadne-codegen
# Source: openhexa/graphql/queries.graphql

from typing import List

from pydantic import Field

from .base_model import BaseModel
from .enums import PipelineError


class DeletePipeline(BaseModel):
    delete_pipeline: "DeletePipelineDeletePipeline" = Field(alias="deletePipeline")


class DeletePipelineDeletePipeline(BaseModel):
    success: bool
    errors: List[PipelineError]


DeletePipeline.model_rebuild()
