# Generated by ariadne-codegen
# Source: openhexa/graphql/queries.graphql

from typing import List

from pydantic import Field

from .base_model import BaseModel
from .enums import DeleteConnectionError


class DeleteConnection(BaseModel):
    delete_connection: "DeleteConnectionDeleteConnection" = Field(
        alias="deleteConnection"
    )


class DeleteConnectionDeleteConnection(BaseModel):
    success: bool
    errors: List[DeleteConnectionError]


DeleteConnection.model_rebuild()
