# Generated by ariadne-codegen
# Source: openhexa/graphql/queries.graphql

from typing import List

from pydantic import Field

from .base_model import BaseModel
from .enums import PipelineTemplateError


class DeletePipelineTemplate(BaseModel):
    delete_pipeline_template: "DeletePipelineTemplateDeletePipelineTemplate" = Field(
        alias="deletePipelineTemplate"
    )


class DeletePipelineTemplateDeletePipelineTemplate(BaseModel):
    success: bool
    errors: List[PipelineTemplateError]


DeletePipelineTemplate.model_rebuild()
