# Generated by ariadne-codegen
# Source: openhexa/graphql/queries.graphql

from typing import List, Optional

from pydantic import Field

from .base_model import BaseModel
from .enums import PipelineError


class CreatePipeline(BaseModel):
    create_pipeline: "CreatePipelineCreatePipeline" = Field(alias="createPipeline")


class CreatePipelineCreatePipeline(BaseModel):
    success: bool
    errors: List[PipelineError]
    pipeline: Optional["CreatePipelineCreatePipelinePipeline"]


class CreatePipelineCreatePipelinePipeline(BaseModel):
    code: str


CreatePipeline.model_rebuild()
CreatePipelineCreatePipeline.model_rebuild()
