# Generated by ariadne-codegen
# Source: openhexa/graphql/queries.graphql

from typing import List, Optional

from pydantic import Field

from .base_model import BaseModel
from .enums import UpdateDatasetError


class UpdateDataset(BaseModel):
    update_dataset: "UpdateDatasetUpdateDataset" = Field(alias="updateDataset")


class UpdateDatasetUpdateDataset(BaseModel):
    success: bool
    errors: List[UpdateDatasetError]
    dataset: Optional["UpdateDatasetUpdateDatasetDataset"]


class UpdateDatasetUpdateDatasetDataset(BaseModel):
    slug: str
    name: str
    description: Optional[str]


UpdateDataset.model_rebuild()
UpdateDatasetUpdateDataset.model_rebuild()
