# Generated by ariadne-codegen
# Source: openhexa/graphql/queries.graphql

from typing import List, Optional

from pydantic import Field

from .base_model import BaseModel
from .enums import CreateDatasetError


class CreateDataset(BaseModel):
    create_dataset: "CreateDatasetCreateDataset" = Field(alias="createDataset")


class CreateDatasetCreateDataset(BaseModel):
    success: bool
    errors: List[CreateDatasetError]
    dataset: Optional["CreateDatasetCreateDatasetDataset"]


class CreateDatasetCreateDatasetDataset(BaseModel):
    slug: str
    name: str


CreateDataset.model_rebuild()
CreateDatasetCreateDataset.model_rebuild()
