# Generated by ariadne-codegen
# Source: openhexa/graphql/queries.graphql

from typing import List

from pydantic import Field

from .base_model import BaseModel
from .enums import DeletePipelineVersionError


class DeletePipelineVersion(BaseModel):
    delete_pipeline_version: "DeletePipelineVersionDeletePipelineVersion" = Field(
        alias="deletePipelineVersion"
    )


class DeletePipelineVersionDeletePipelineVersion(BaseModel):
    success: bool
    errors: List[DeletePipelineVersionError]


DeletePipelineVersion.model_rebuild()
