# Generated by ariadne-codegen
# Source: openhexa/cli/graphql/queries.graphql

from pydantic import Field

from .base_model import BaseModel


class Me(BaseModel):
    me: "MeMe"


class MeMe(BaseModel):
    has_two_factor_enabled: bool = Field(alias="hasTwoFactorEnabled")


Me.model_rebuild()
