# Generated by ariadne-codegen
# Source: openhexa/graphql/queries.graphql

from typing import Any, List, Optional

from pydantic import Field

from .base_model import BaseModel
from .enums import UpdateWorkspaceError


class UpdateWorkspace(BaseModel):
    update_workspace: "UpdateWorkspaceUpdateWorkspace" = Field(alias="updateWorkspace")


class UpdateWorkspaceUpdateWorkspace(BaseModel):
    success: bool
    workspace: Optional["UpdateWorkspaceUpdateWorkspaceWorkspace"]
    errors: List[UpdateWorkspaceError]


class UpdateWorkspaceUpdateWorkspaceWorkspace(BaseModel):
    slug: str
    name: str
    description: Optional[str]
    configuration: Optional[Any]
    countries: List["UpdateWorkspaceUpdateWorkspaceWorkspaceCountries"]


class UpdateWorkspaceUpdateWorkspaceWorkspaceCountries(BaseModel):
    code: str
    alpha_3: str = Field(alias="alpha3")
    name: str


UpdateWorkspace.model_rebuild()
UpdateWorkspaceUpdateWorkspace.model_rebuild()
UpdateWorkspaceUpdateWorkspaceWorkspace.model_rebuild()
