# Generated by ariadne-codegen
# Source: openhexa/graphql/queries.graphql

from typing import Any, List, Optional

from pydantic import Field

from .base_model import BaseModel


class Organization(BaseModel):
    organization: Optional["OrganizationOrganization"]


class OrganizationOrganization(BaseModel):
    id: Any
    name: str
    short_name: Optional[str] = Field(alias="shortName")
    workspaces: "OrganizationOrganizationWorkspaces"
    permissions: "OrganizationOrganizationPermissions"


class OrganizationOrganizationWorkspaces(BaseModel):
    items: List["OrganizationOrganizationWorkspacesItems"]


class OrganizationOrganizationWorkspacesItems(BaseModel):
    slug: str
    name: str
    countries: List["OrganizationOrganizationWorkspacesItemsCountries"]


class OrganizationOrganizationWorkspacesItemsCountries(BaseModel):
    code: str


class OrganizationOrganizationPermissions(BaseModel):
    create_workspace: bool = Field(alias="createWorkspace")
    archive_workspace: bool = Field(alias="archiveWorkspace")


Organization.model_rebuild()
OrganizationOrganization.model_rebuild()
OrganizationOrganizationWorkspaces.model_rebuild()
OrganizationOrganizationWorkspacesItems.model_rebuild()
