# Generated by ariadne-codegen
# Source: openhexa/cli/graphql/queries.graphql

from typing import Optional

from .base_model import BaseModel


class GetWorkspace(BaseModel):
    workspace: Optional["GetWorkspaceWorkspace"]


class GetWorkspaceWorkspace(BaseModel):
    name: str
    slug: str


GetWorkspace.model_rebuild()
