# Generated by ariadne-codegen
# Source: openhexa/graphql/queries.graphql

from typing import Any, List

from .base_model import BaseModel


class Organizations(BaseModel):
    organizations: List["OrganizationsOrganizations"]


class OrganizationsOrganizations(BaseModel):
    id: Any
    name: str
    workspaces: "OrganizationsOrganizationsWorkspaces"


class OrganizationsOrganizationsWorkspaces(BaseModel):
    items: List["OrganizationsOrganizationsWorkspacesItems"]


class OrganizationsOrganizationsWorkspacesItems(BaseModel):
    slug: str
    name: str


Organizations.model_rebuild()
OrganizationsOrganizations.model_rebuild()
OrganizationsOrganizationsWorkspaces.model_rebuild()
