# Generated by ariadne-codegen
# Source: openhexa/graphql/queries.graphql

from typing import List

from pydantic import Field

from .base_model import BaseModel
from .enums import AddToFavoritesError


class AddWebappToFavorites(BaseModel):
    add_to_favorites: "AddWebappToFavoritesAddToFavorites" = Field(
        alias="addToFavorites"
    )


class AddWebappToFavoritesAddToFavorites(BaseModel):
    success: bool
    errors: List[AddToFavoritesError]


AddWebappToFavorites.model_rebuild()
