# Generated by ariadne-codegen
# Source: openhexa/graphql/queries.graphql

from typing import List

from pydantic import Field

from .base_model import BaseModel
from .enums import DeleteWebappError


class DeleteWebapp(BaseModel):
    delete_webapp: "DeleteWebappDeleteWebapp" = Field(alias="deleteWebapp")


class DeleteWebappDeleteWebapp(BaseModel):
    success: bool
    errors: List[DeleteWebappError]


DeleteWebapp.model_rebuild()
