# Generated by ariadne-codegen
# Source: openhexa/graphql/queries.graphql

from typing import List

from pydantic import Field

from .base_model import BaseModel
from .enums import UpdateWebappError


class UpdateWebapp(BaseModel):
    update_webapp: "UpdateWebappUpdateWebapp" = Field(alias="updateWebapp")


class UpdateWebappUpdateWebapp(BaseModel):
    success: bool
    errors: List[UpdateWebappError]


UpdateWebapp.model_rebuild()
