# Generated by ariadne-codegen
# Source: openhexa/graphql/queries.graphql

from typing import List

from pydantic import Field

from .base_model import BaseModel
from .enums import PipelineRecipientError


class AddPipelineRecipient(BaseModel):
    add_pipeline_recipient: "AddPipelineRecipientAddPipelineRecipient" = Field(
        alias="addPipelineRecipient"
    )


class AddPipelineRecipientAddPipelineRecipient(BaseModel):
    success: bool
    errors: List[PipelineRecipientError]


AddPipelineRecipient.model_rebuild()
